//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.*

#include "GamePreferencesDialog.h"
#include <pglabel.h>
#include <pgfilearchive.h>
#include "PG_FileDialog.h"
#include "GameScenario.h"
#include "MapConfDialog.h"
#include "player_preferences.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

GamePreferencesDialog::GamePreferencesDialog(PG_Widget* parent, PG_Rect rect)
	:PG_ThemeWidget(parent, rect)
{
    debug("GamePreferencesDialog()");

    d_l_type = new PG_Label(this, PG_Rect(40, 10, 100, 20), "Type");
    d_l_name = new PG_Label(this, PG_Rect(145, 10, 100, 20), "Name");
    d_l_armyset = new PG_Label(this, PG_Rect(255, 10, 100, 20), "Armyset");
    d_l_start = new PG_Label(this, PG_Rect(358, 10, 100, 20), "Start");

    // predefined names
    vector<string> nameList;
    nameList.push_back("Michael");
    nameList.push_back("Susi");
    nameList.push_back("Ulf");
    nameList.push_back("James");
    nameList.push_back("Vibhu");
    nameList.push_back("Tobias");
    nameList.push_back("Linus");
    nameList.push_back("Alan");
	for (int i = 0; i < 8; ++i)
	{
        PG_Rect p(10, 40 + i*35, 400, 25);
        Player_preferences::Type type = Player_preferences::ANY;
        if (i == 0) type = Player_preferences::HUMAN;
        if (i == 1) type = Player_preferences::HUMAN_OR_COMPUTER;
        d_player_preferences[i] = new Player_preferences(type, nameList[i], this, p);
        d_player_preferences[i]->playerDataChanged.connect(
                (SigC::slot(*this, &GamePreferencesDialog::slotPlayerDataChanged)));
	}

	d_b_random = new PG_Button(this, 0, PG_Rect(Width() - 310 , 10, 100, 25), "Random");
    d_b_random->SetToggle(true);
	d_b_load = new PG_Button(this, 1, PG_Rect(Width() - 210, 10, 100, 25), "Load Map");
    d_b_load->SetToggle(true);
    d_edit = new PG_LineEdit(this, PG_Rect(Width() - 310, 60, 200, 25));
    d_edit->SetText("random.map");
	d_b_browse = new PG_Button(this, 2, PG_Rect(Width() - 100, 60, 90, 25), "Browse");
    d_water = new TerrainConfig(this, PG_Rect(Width() - 310, 60, 300, 20),
            "Water", 0, 90, 30);
    d_swamp = new TerrainConfig(this, PG_Rect(Width() - 310, 90, 300, 20),
            "Swamp", 0, 90, 20);
    d_forest = new TerrainConfig(this, PG_Rect(Width() - 310, 120, 300, 20),
            "Forest", 0, 90, 20);
    d_hills = new TerrainConfig(this, PG_Rect(Width() - 310, 150, 300, 20),
            "Hills", 0, 90, 10);
    d_mountains = new TerrainConfig(this, PG_Rect(Width() - 310, 180, 300, 20),
            "Mountains", 0, 50, 5);
    d_cities = new TerrainConfig(this, PG_Rect(Width() - 310, 210, 300, 20),
            "Cities", 10, 99, 40);

    d_b_ok = new PG_Button(this, 3, PG_Rect(10, Height() - 40, Width() - 20, 30), "OK");

    // if the previous map is still available use this it as default setting
	if (PG_FileArchive::Exists("random.map"))
	{
        GamePreferencesDialogType = GameScenario::LOADED;
	}
    else
    {
        GamePreferencesDialogType = GameScenario::RANDOM;
    }

	d_b_random->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::randomClicked);
	d_b_load->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::loadClicked);
	d_b_browse->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::browseClicked);
	d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&GamePreferencesDialog::okClicked);
}

GamePreferencesDialog::~GamePreferencesDialog()
{
    for (int i = 0; i < 8; i++)
        delete d_player_preferences[i];
    delete d_l_type;
    delete d_l_name;
    delete d_l_armyset;
    delete d_l_start;
    delete d_b_random;
    delete d_b_load;
    delete d_b_browse;
    delete d_b_ok;
    delete d_water;
    delete d_swamp;
    delete d_forest;
    delete d_hills;
    delete d_mountains;
    delete d_cities;
}

void GamePreferencesDialog::initGUI()
{
    debug("initGUI");
    if (GamePreferencesDialogType == GameScenario::LOADED)
    {
        d_water->hide();
        d_swamp->hide();
        d_forest->hide();
        d_hills->hide();
        d_mountains->hide();
        d_cities->hide();
        d_edit->Show();
        d_b_browse->Show();
        d_b_load->SetPressed(true);
        d_b_random->SetPressed(false);
    }
    else
    {
        d_water->show();
        d_swamp->show();
        d_forest->show();
        d_hills->show();
        d_mountains->show();
        d_cities->show();
        d_edit->Hide();
        d_b_browse->Hide();
        d_b_load->SetPressed(false);
        d_b_random->SetPressed(true);
    }
}

PARAGUI_CALLBACK(GamePreferencesDialog::browseClicked)
{
	PG_FileDialog fd;
	fd.Show();
	fd.RunModal();
    fd.Hide();

	d_fileName = fd.getSelectedFile();
	if (!d_fileName.empty())
	{
        d_edit->SetText(d_fileName.c_str());
	}
	return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::randomClicked)
{
	GamePreferencesDialogType = GameScenario::RANDOM;
    initGUI();
    return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::loadClicked)
{
	GamePreferencesDialogType = GameScenario::LOADED;
    initGUI();
	return true;
}

PARAGUI_CALLBACK(GamePreferencesDialog::okClicked)
{
	SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

void GamePreferencesDialog::slotPlayerDataChanged()
{
    debug("slotPlayerDataChanged()");
    playerDataChanged.emit();
}

int GamePreferencesDialog::noPlayers()
{
	int n = 0;
	
	for (int i = 0; i < 8; i++)
		if (d_player_preferences[i]->isActive())
			n++;
	
	return n;
}

bool GamePreferencesDialog::getData(int i, Playerstruct* ps)
{
	if (!d_player_preferences[i]->isActive()) return false;

	ps->name = d_player_preferences[i]->get_name();
	ps->armyset = d_player_preferences[i]->get_armyset();
	ps->computer = d_player_preferences[i]->isComputer();

    if (d_player_preferences[i]->isComputer())
        ps->type = Player::AI_FAST;
    else
        ps->type = Player::HUMAN;

	switch(i)
	{
		case 0:ps->color.r = ps->color.g = 0; ps->color.b = 255; break;
		case 1:ps->color.r = 255; ps->color.g = ps->color.b = 0; break;
		case 2:ps->color.r = 0; ps->color.g = 255; ps->color.b = 0; break;
		case 3:ps->color.r = ps->color.g = ps->color.b = 190; break;
		case 4:ps->color.r = 255; ps->color.g = 0; ps->color.b = 255;break;
		case 5:ps->color.r = ps->color.g = 255; ps->color.b = 0; break;
		case 6:ps->color.r = ps->color.g = ps->color.b = 255; break;
		case 7:ps->color.r = ps->color.g = ps->color.b = 0; break;
	}

	return true;
}
// End of file
