//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include "GameScenario.h"
#include "MapCreationDialog.h"
#include "GamePreferencesDialog.h"
#include "MapGenerator.h"
#include "armyset.h"
#include "playerlist.h"
#include "citylist.h"
#include "city.h"
#include "ruin.h"
#include "armysetlist.h"
#include "stacklist.h"
#include "mapdef.h"
#include "player.h"
#include "Configuration.h"
#include "real_player.h"
#include "ai_dummy.h"
#include "ai_fast.h"
#include "file.h"
#include "counter.h"
// TBD - Port to SDL_net
//#include "Server.h"
//#include "Client.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

GameScenario::GameScenario(GameType gametype, string ip, int port)
	:d_gamePreferencesDialog(0), d_gameType(gametype), d_turns(0)
{
	debug("GameScenario()")
	
	armysetlist = new Armysetlist();
	citylist = new Citylist();
	ruinlist = new Ruinlist();
    templeList = new TempleList();
	playerlist = new Playerlist();
    mapdef = new Mapdef();
	stacklist = new Stacklist();
    fl_counter = new FL_Counter();

    if (d_gameType == MULTI_PLAYER_HOST)
    {
        /* TBD
        d_server = new Server(port);
        d_server->acceptClients();
        */
    }
    else if (d_gameType == MULTI_PLAYER_CLIENT)
    {
        /* TBD
        d_client = new Client(ip, port);
        */
    }

	initGame();
}

// savegame is a filename with absolute path!

GameScenario::GameScenario(string savegame, bool& broken)
{
    broken = false;
    XML_Helper helper(savegame, ios::in);

    armysetlist = new Armysetlist();

    helper.registerTag("scenario", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("map", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("playerlist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("citylist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("templelist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("ruinlist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("stacklist", SigC::slot((*this), &GameScenario::load));
    helper.registerTag("counter", SigC::slot((*this), &GameScenario::load));

    //now parse the document and close the file afterwards
    if (!helper.parse())
    {
        broken = true;
    }

    helper.close();
}

GameScenario::~GameScenario()
{
    delete fl_counter;
	delete mapdef;
	delete citylist;
	delete ruinlist;
	delete templeList;
	delete playerlist;
	delete armysetlist;
	delete stacklist;
    delete d_gamePreferencesDialog;

    // still some delete's missing?
} 

void GameScenario::addPlayer(string name, string armyset,
        SDL_Color color, Player::TYPE type)
{
    //four cases up to now: human player (HUMAN), neutral  player 
    //(NEUTRAL), standard computer player (AI_FAST) or dummy computer
    //(AI_DUMMY)
    debug("adding player " << name << " to list");

    Player* myplayer;
    if (type == Player::HUMAN)
    {
		myplayer = new RealPlayer(name, armyset, color);
		playerlist->push_back(myplayer);
	} 
    else if (type == Player::NEUTRAL)
	{
		if (playerlist->getNeutral() != 0)
		{
			cerr <<"attempt to insert neutral player twice\nexiting\n";
			exit(-1);
		}
		myplayer = new AI_Dummy(name, armyset, color);
		myplayer->setType(Player::NEUTRAL);
		playerlist->push_back(myplayer);
		playerlist->setNeutral(myplayer);
	}
    else if (type == Player::AI_DUMMY)
	{
		myplayer = new AI_Dummy(name, armyset, color);
		playerlist->push_back(myplayer);
	}
    else if (type == Player::AI_FAST)
	{
		myplayer = new AI_Fast(name, armyset, color);
		playerlist->push_back(myplayer);
	}

	//load player's armyset
	if (!armysetlist->loadArmyset(armyset))
	{
        cerr << "GameScenario: couldn't load armyset: " << armyset << endl;
        cerr << "Exiting FreeLords" << endl;
        exit(-1);
    }
}

void GameScenario::initGame()
{
    debug("initGame()");

    MapType maptype = RANDOM;

    //always set up neutral player
    SDL_Color gray;
    gray.r = gray.g = gray.b = 150;
    addPlayer("Neutral", "default", gray, Player::NEUTRAL);

    if (!Configuration::s_testing)
    {
        d_gamePreferencesDialog = new GamePreferencesDialog(0, PG_Rect(0, 0,
                    PG_Application::GetScreenWidth(),
                    PG_Application::GetScreenHeight()));

        /* TBD
        if (d_gameType == MULTI_PLAYER_CLIENT && d_client)
        {
            dialog.playerDataChanged.connect(
                    SigC::slot(*d_client, &Client::sendPlayerInfo));
        }
        */
        d_gamePreferencesDialog->Show();
        d_gamePreferencesDialog->initGUI();
        d_gamePreferencesDialog->RunModal();
        d_gamePreferencesDialog->Hide();
        maptype = d_gamePreferencesDialog->getMapType();

        Playerstruct ps;
        for (int i = 0; i < 8; i++)
        {
            if (d_gamePreferencesDialog->getData(i, &ps))
            {
                Player::TYPE type = (ps.computer ? 
                        Player::AI_FAST : Player::HUMAN);
                addPlayer(ps.name, ps.armyset, ps.color, type);
            }
        }
    }
    else
    {
        // Do a test scenario with 2 computer player here
        SDL_Color color;
        color.r = color.g = 0; color.b = 255;
        addPlayer("Michael", "default", color, Player::AI_FAST);

        color.r = 255; color.g = color.b = 0;
        addPlayer("Susi", "default", color, Player::AI_FAST);
    }	

    // show the progress of map creation
    MapCreationDialog mcd(0,
            PG_Rect((PG_Application::GetScreenWidth() - 140) / 2,
            (PG_Application::GetScreenHeight() - 140) / 2, 140, 120));
    mcd.setProgress(0);
    mcd.Show();

    // now initialize it
    if (maptype == LOADED)
    {
        mapdef->random();
        // TBD mapdef->load(d_map.get_filename());
    }
    else //maptype == RANDOM
    {
        // why not have mapdef handle generated maps directly without any
        // files? -> [MB] OK, but players should be able to save the map if
        // they want!
        MapGenerator mg;
        mg.MakeMap(100, 100);
        mg.DumpMap("random.map");
        mapdef->random();
    }

    //TBD
    //mcd.setText("Smoothing map");
    //mcd.setProgress(60);
    //mapdef->smoothAll();

    switch (d_gameType)
    {
        case SINGLE_PLAYER:
            {
                initSinglePlayer();
                break;
            }
        case SCENARIO:
            {	
                initScenario();
                break;
            }
        case MULTI_PLAYER_HOST:
            {
                initMultiPlayer();
                break;
            }
        case MULTI_PLAYER_CLIENT:
            {
                initMultiPlayer();
                break;
            }
    }
}

bool GameScenario::isMultiPlayer() const
{
	return (d_gameType == MULTI_PLAYER_HOST) || (d_gameType == MULTI_PLAYER_CLIENT);
}

void GameScenario::initSinglePlayer()
{
	debug("initSinglePlayer()");

	checkInitialisation();
	assignPlayers();
	produceFirstArmy();
	misc();

	//quick hack: add a hero to each player
	//we know that some cities already belong to the players so we use
	//them to set up our hero
	for (Citylist::iterator c_it = citylist->begin(); c_it != citylist->end();
				c_it++)
	{
		if (c_it->getPlayer()->getName() != "Neutral")
			c_it->addHero();
			//Ok, all heroes are now called Demonlord, but that will
			//do it for now
	}
}

void GameScenario::initScenario()
{
	//scenarios differ from simple games by the fact that players, player-owned
	//cities etc. are already given at game start and (later) by different
	//objectives, and are also part of campaigns. Currently, you would
	//hardly benefit from scenarios, so they aren't implemented yet :).
}

void GameScenario::initMultiPlayer()
{
	/*TBD
	debug("initMultiPlayer()");
	client->wait_for_map();
	mapdef->random();

	client->wait_for_playerorder();
	load_armyset();

	if (client->is_first())
	{
		assign_players();
		produce_first_army();
	}

	//we just assume, the map from the server is already smoothed, so we
	//don't need to do so
	*/
}

// Create a stack in each city with the weakest army the city can produce

void GameScenario::produceFirstArmy()
{
	debug("produceFirstArmy()");
	for (Citylist::iterator it = citylist->begin(); it != citylist->end(); it++)
	{
 		// This former default will become configuration "Neutral Hard"
 		// (*it).set_production(0);
 		// (*it).produce_army();
 
 		// This new default will become configuration "Neutral Medium"
 		(*it).produceLargestArmy();
 		if ( (*it).getPlayer()->getName() == "Neutral" )
 		{
 			(*it).setProduction(-1);
 		}
 		else
 		{
 			(*it).setProduction(0);
 		}
 
 		// This will become configuration "Neutral Easy"
 		// (*it).produceArmy();
 		// (*it).setProduction(-1);
	}
}

// assign each player a capital

void GameScenario::assignPlayers()
{
	debug("assignPlayers()");
 
 	int skip_cities = (citylist->size() - 1) / (playerlist->size() - 1) - 1;
 	int skipped = 0;
 	Playerlist::iterator it = ++(playerlist->begin());  // don't assign to neutral player

 	for (Citylist::iterator c_it = citylist->begin(); c_it != citylist->end(); c_it++)
 	{
 		if ( ++skipped == skip_cities && it != playerlist->end())
 		{
            (*c_it).setPlayer(*it);
            it++;
            skipped = 0;
 		}
        else
        {
            (*c_it).setPlayer(playerlist->getNeutral());
        }
	}
}

void GameScenario::checkInitialisation()
{
	debug("checkInitialisation()");
	if (citylist->size() == 0)
	{
		cerr << "ERROR: There is no city on this map! Please use the random map feature\n";
		exit(-1);
	}	       
}

void GameScenario::nextTurn()
{
	if (++d_turns != 0)
	{
		citylist->nextTurn();
		stacklist->nextTurn();
	}
}

//a function as quick hack; we need to do some rework anyway,so don't bother
void GameScenario::misc()
{
	debug("misc()")
	//create a strong stack

	for (Ruinlist::iterator it = ruinlist->begin(); it != ruinlist->end(); it++)
	{
		Stack* keeper = new Stack(0, (*it).getPos());
		keeper->push_back(new Army((*armysetlist->begin()).second, 9));
		(*it).set_occupant(keeper);
	}
}

bool GameScenario::saveGame(string filename)
{
	bool retval = true;

	XML_Helper helper(filename, ios::out);

	//start writing
    retval &= helper.begin(FREELORDS_SAVEGAME_VERSION);
	retval &= helper.openTag("freelords");
	
	//save the private GameScenario data first
	retval &= helper.openTag("scenario");
	retval &= helper.saveData("name", d_name);
	retval &= helper.saveData("type", d_gameType);
	retval &= helper.saveData("turn", d_turns);
	retval &= helper.closeTag();
	if (!retval)
	{
        std::cerr <<"GameScenario: Something went wrong with saving.\n";
		return false;
	}
	
	//if retval is still true it propably doesn't change throughout the rest
	//now save the single object's data
    retval &= fl_counter->save(&helper);
	retval &= mapdef->save(&helper);
	retval &= playerlist->save(&helper);
	retval &= citylist->save(&helper);
	retval &= templeList->save(&helper);
	retval &= ruinlist->save(&helper);
	retval &= stacklist->save(&helper);

	retval &= helper.closeTag();

	helper.close();

	if (retval)
    {
		return true;
    }
		
    std::cerr <<"GameScenario: Something went wrong with saving.\n";
	return false;
}

bool GameScenario::load(string tag, XML_Helper* helper)
{
    int i;
    
    if (tag == "scenario")
    {
        if (helper->getVersion() != FREELORDS_SAVEGAME_VERSION)
        {
            cerr <<"savefile has wrong version, we want ";
            std::cerr <<FREELORDS_SAVEGAME_VERSION <<",\n";
            cerr <<"savefile offers " <<helper->getVersion() <<".\n";
            return false;
        }
    
        debug("loading scenario")
        helper->getData(d_turns, "turn");
        helper->getData(d_name, "name");
        helper->getData(i, "type");
        d_gameType = static_cast<GameType>(i);

        //TODO: for later when it becomes crucial: deal with loading of
        //something else than simple games
        return true;
    }

    if (tag == "counter")
    {
        debug("loading counter")
        fl_counter = new FL_Counter(helper);
        return true;
    }

    if (tag == "map")
    {
        debug("loading map")
        mapdef = new Mapdef(helper);
        return true;
    }

    if (tag == "playerlist")
    {
        debug("loading players");
        playerlist = new Playerlist(helper);
        return true;
    }

    if (tag == "citylist")
    {
        debug("loading cities")

        citylist = new Citylist(helper);
        return true;
    }

    if (tag == "templelist")
    {
        debug("loading temples")
        templeList = new TempleList(helper);
        return true;
    }

    if (tag == "ruinlist")
    {
        debug("loading ruins")
        ruinlist = new Ruinlist(helper);
        return true;
    }

    if (tag == "stacklist")
    {
        debug("loading stacks")
        stacklist =new Stacklist(helper);
        return true;
    }

    return false;
}
