//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "mapdef.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include "citylist.h"
#include "city.h"
#include "ruin.h"
#include "temple.h"
#include "playerlist.h"
#include "file.h"

using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Mapdef* mapdef;

Mapdef::Mapdef()
{
    d_tileSet = new TileSet("default");
}

Mapdef::Mapdef(XML_Helper* helper)
{
    d_tileSet = new TileSet("default");
	
	string types;
	string buildings;
	helper->getData(types, "types");
	helper->getData(buildings, "buildings");

	for (int j = 0; j < 100; j++)
    {
		for (int i = 0; i < 100; i++)
		{
            //due to the circumstances types is a long stream of
            //numbers, so read it character for character (no \n's or so)
			char type = types[j*100 + i];  

            //the chars now hold the ascii representation of the numbers, which
            //we don't want
            type -= '0';
			map[i][j] = new Maptile(d_tileSet, i, j, type);
		}
    }

    // this has to be done seperately, because CITY sets 4 squares (and the
    // other squares are not created at the time we want to set them)!
	for (int j = 0; j < 100; j++)
    {
		for (int i = 0; i < 100; i++)
		{
			char buildingTemp = buildings[j*100 + i];  
            Maptile::Building building = (Maptile::Building)(buildingTemp - '0');
            map[i][j]->setBuilding(building);
            if (building == Maptile::CITY)
            {
                map[i+1][j]->setBuilding(building);
                map[i][j+1]->setBuilding(building);
                map[i+1][j+1]->setBuilding(building);
            }
		}
    }
}

Mapdef::~Mapdef()
{
    delete d_tileSet;

	for (int i = 0; i < 100; i++)
	{
		for (int j = 0; j < 100; j++)
		{
            delete map[i][j];
		}
	}
}

bool Mapdef::fill(MapGenerator* generator)
//basically, this does the same as the random function, but you don't need to
//go the whole way via dumping the map in a file etc.
{
    int xsize, ysize;
    int type;
    char* mymap;

    mymap = generator->getMap(xsize, ysize);

    for (int i = 0; i < xsize; i++)
        for (int j = 0; j < ysize; j++)
        {
            switch (mymap[i+j*xsize])
            {
 				case CHARACTER_WATER:
					type = 1;
					break;
				case CHARACTER_FOREST:
					type = 2; 
					break;
				case CHARACTER_HILLS:
					type = 3;
					break;
				case CHARACTER_MOUNTAINS:
					type = 4;
					break;
				case CHARACTER_SWAMP:
					type = 5;
					break;
				default:
					type = 0;
					break;
            }
			map[i][j] = new Maptile(d_tileSet, i, j, type);
        }

    mymap = generator->getBuildings(xsize, ysize);

    for (int i = 0; i < xsize; i++)
    {
        for (int j = 0; j < ysize; j++)
        {
            switch (mymap[i + j*xsize])
            {
                case CHARACTER_CITY_MINOR:
                case CHARACTER_CITY_MAJOR:
                    map[i][j]->setMoves(1);
                    map[i][j]->setBuilding(Maptile::CITY);
                    break;
                case CHARACTER_RUIN:
                    map[i][j]->setBuilding(Maptile::RUIN);
                    break;
                case CHARACTER_TEMPLE:
                    map[i][j]->setBuilding(Maptile::TEMPLE);
                    break;
            }
        }
    }
    return true;
}


void Mapdef::random()
{
	debug("random()\n");

	char c;		// Single maptile read from file
	int type;

	ifstream map_file("random.map");	// Open file

	for (int j = 0; j < 100; j++)
		for (int i = 0; i < 100; i++)
		{
			map_file >> c;

			switch (c)
			{
				case CHARACTER_WATER:
				{
					type = 1;
					break;
				}
				case CHARACTER_FOREST:
				{
					type = 2; 
					break;
				}
				case CHARACTER_HILLS:
				{
					type = 3;
					break;
				}
				case CHARACTER_MOUNTAINS:
				{
					type = 4;
					break;
				}
				case CHARACTER_SWAMP:
				{
					type = 5;
					break;
				}
				default:
				{
					type = 0;
					break;
				}
			}

			debug("creating maptile[" << i << "][" << j << "]: " << c);
			map[i][j] = new Maptile(d_tileSet, i, j, type);
			//map[i][j]->print_debug_info();
		}
	
	for (int j = 0; j < 100; j++)
	{	
		//remove the final newline
		map_file.get(c);
		for (int i = 0; i < 100; i++)
		{
			map_file.get(c);
			PG_Point p;
			p.x = i;
			p.y = j;

			switch (c)
			{
				case CHARACTER_CITY_MAJOR:
				{
					City city(p);
					city.setRandomArmytypes();
					citylist->push_back(city);

					// change settings for the maptiles the city is on
					for (int a = 0; a < 2; ++a)
						for (int b = 0; b < 2; ++b)
						{
							map[p.x+a][p.y+b]->setMoves(1);
							map[p.x+a][p.y+b]->setBuilding(Maptile::CITY);
						}
					break;
				}
				case CHARACTER_RUIN:
				{
					ruinlist->push_back(Ruin(p));
					map[p.x][p.y]->setBuilding(Maptile::RUIN);
					break;
				}
				case CHARACTER_TEMPLE:
				{

					templeList->push_back(Temple(p));
					map[p.x][p.y]->setBuilding(Maptile::TEMPLE);
					break;
				}
			}
		}
	}

	//finally, give the cities names (appended in the random.map file)
	//TODO: this code trusts the map file, this is bad behaviour

	//remove trailing newline
	map_file.get(c);

	char buffer[100];
	for (Citylist::iterator it = citylist->begin(); it != citylist->end(); it++)
	{
		map_file.getline(buffer, 100);
		(*it).setName(string(buffer));
		debug("added city "<<buffer<<endl)
	}

}


//when changing anything on the save function, never forget to do the same
//changes in CreateScenario!!!
bool Mapdef::save(XML_Helper* helper)
{
	bool retval = true;

	stringstream types;
    stringstream buildings;

	types << endl;
	buildings << endl;
	for (int i = 0; i < 100; i++)
	{
		for (int j = 0; j < 100; j++)
		{
			types << map[j][i]->getType();
			buildings << map[j][i]->getBuilding();
		}
		buildings << endl;
		types << endl;
	}


	retval &= helper->openTag("map");
	retval &= helper->saveData("types", types.str());
	retval &= helper->saveData("buildings", buildings.str());
	retval &= helper->closeTag();
	return retval;
}

// End of file
