//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "splash.h"
#include <iostream>
#include <SDL_image.h>
#include "MainWindow.h"
#include "defs.h"
#include "armysetlist.h"
#include "citylist.h"
#include "stacklist.h"
#include "playerlist.h"
#include "Configuration.h"
#include "MultiPlayerModeDialog.h"
#include "PG_FileDialog.h"
#include "file.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

Splash::Splash(PG_Application* app, const PG_Rect& rect)
	:PG_ThemeWidget(0, rect), d_mainWindow(0)
{
    d_gameScenario = 0;
    d_app = app;
	d_background = File::getMiscPicture("splash_screen.jpg", false);
	SetBackground(d_background, 2);
	
	d_b_campaign = new PG_Button(this, 0,
            PG_Rect(rect.w - 210, rect.h - 180, 180, 30), "New Campaign");
	d_b_singleplayer_game = new PG_Button(this, 1,
            PG_Rect(rect.w - 210, rect.h - 145, 180, 30), "New Game");
	d_b_multiplayer_game = new PG_Button(this, 2,
            PG_Rect(rect.w - 210, rect.h - 110, 180, 30), "New Multiplayer Game");
	d_b_load_game = new PG_Button(this, 3,
            PG_Rect(rect.w - 210, rect.h - 75, 180, 30), "Load Game");
	d_b_quit = new PG_Button(this, 4,
            PG_Rect(rect.w - 210, rect.h - 40, 180, 30), "Quit");

	d_b_campaign->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_campaign_clicked);
	d_b_singleplayer_game->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_singleplayer_game_clicked);
	d_b_multiplayer_game->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_multiplayer_game_clicked);
	d_b_load_game->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_load_game_clicked);
	d_b_quit->SetEventObject(MSG_BUTTONCLICK, this,
            (MSG_CALLBACK_OBJ)&Splash::b_quit_clicked);
}

Splash::~Splash()
{
    SDL_FreeSurface(d_background);
    delete d_gameScenario;
	clear_data();
	delete d_b_campaign;
	delete d_b_singleplayer_game;
	delete d_b_multiplayer_game;
	delete d_b_load_game;
	delete d_b_quit;
}

PARAGUI_CALLBACK(Splash::b_quit_clicked)
{
	d_app->Quit();
	return true;
}

PARAGUI_CALLBACK(Splash::b_singleplayer_game_clicked)
{
	clear_data();
	newGame(GameScenario::SINGLE_PLAYER);
	return true;
}

void Splash::newGame(GameScenario::GameType gameType, std::string ip, int port)
{
    d_gameScenario = new GameScenario(gameType, ip, port);

    playerlist->sgameWon.connect(SigC::slot((*this), &Splash::gameFinished));

	Hide();
		
	d_mainWindow = new MainWindow(d_gameScenario, PG_Rect(0, 0, Width(), Height()));
    d_mainWindow->squitting.connect(SigC::slot((*this), &Splash::gameFinished));
	d_mainWindow->Show();
	d_mainWindow->start_game();
}

PARAGUI_CALLBACK(Splash::b_multiplayer_game_clicked)
{
	clear_data();

    MultiPlayerModeDialog dialog(this, PG_Rect(20, 20, 350, 230));
    dialog.Show();
    dialog.RunModal();
    dialog.Hide();

    if (dialog.getResult())
    {
        if (dialog.getMode())
        {
            newGame(GameScenario::MULTI_PLAYER_HOST, "", dialog.getPort());
        }
        else
        {  
            newGame(GameScenario::MULTI_PLAYER_CLIENT, dialog.getIP(), dialog.getPort());
        }
    }

	return true;
}

PARAGUI_CALLBACK(Splash::b_load_game_clicked)
{
	clear_data();

    PG_FileDialog opendialog(0, PG_FileDialog::PG_OPEN, true,
            Configuration::s_savePath, "*.sav", "FreeLords savegames");
    opendialog.Show();
    opendialog.RunModal();
    std::string savegame = opendialog.getSelectedFile();

    //no file selected
    if (savegame.empty())
    {
        return true;
    }
    
    bool broken;
    d_gameScenario = new GameScenario(savegame, broken);

    if (broken)
    {
        std::cerr <<"Couldn't load savegame " <<savegame <<". Exiting.\n";
        exit(-1);
    }

    playerlist->sgameWon.connect(SigC::slot((*this), &Splash::gameFinished));

	Hide();
		
	d_mainWindow = new MainWindow(d_gameScenario, PG_Rect(0, 0, Width(), Height()));
	d_mainWindow->squitting.connect(SigC::slot((*this), &Splash::gameFinished));
    d_mainWindow->Show();
	d_mainWindow->loadGame(savegame);

	return true;
}

PARAGUI_CALLBACK(Splash::b_campaign_clicked)
{
	clear_data();
	std::cerr << "not implemented yet\n";
	return true;
}

void Splash::clear_data()
{
    delete d_mainWindow;
}

void Splash::gameFinished()
{
	debug("gameFinished()");

	//the memory for gameScenario and mainWindow is freed as soon as a
	//new game is started

	d_mainWindow->Hide();
	Show();
}
