//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef STACKLIST_H
#define STACKLIST_H

#include <list>
#include <pgmessageobject.h>
#include <pgeventobject.h>
#include <sigc++/object_slot.h>

class City;
class Stack;
class XML_Helper;

class Stacklist : public std::list<Stack*>, public PG_EventObject, public SigC::Object
{
	public:
		// CREATORS
		Stacklist();
        Stacklist(XML_Helper* helper);
		~Stacklist();

		// MANIPULATORS
		void setActivestack(Stack* activestack) {d_activestack = activestack;}
		Stack* setNext();
		bool canJoin(Stack* s, Stack* d);
        // do all stuff needed to proceed to next turn
		void nextTurn();
        // remove this player's stacks
		PARAGUI_CALLBACK(removeAll);
		std::vector<Stack*> defendersInCity(City* c);
        
		bool save(XML_Helper* helper);
        bool load(std::string tag, XML_Helper* helper);
		
		// ACCESSORS
		Stack* getActivestack() const {return d_activestack;}
		Stack* getObjectAt(int x, int y) const;
		Stack* getObjectAt(PG_Point point) const;
        //look if another stack occupies the same point as stack "s"
        //(need to force joining of stacks when moving)
		Stack* getAmbiguity(Stack* s);

		//remove methods which also frees the memory of the pointers
		void flClear();
		iterator flErase(iterator object);
		bool flRemove(Stack* object);

	private:
		Stack* d_activestack;
};

extern Stacklist* stacklist;

#endif // STACKLIST_H

// End of file
